// 
// Copyright (C) 2006, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// NOT FOR PUBLIC DISTRIBUTION!!!
	/// </summary>
	[Description("The RSX is a smooth, low lag price-following oscillator that ranges between 0 and 100.")]
	public class RSX : Indicator
	{
		#region Variables
		private DataSeries					priceSeries;
		private DataSeries					avgDown;
		private DataSeries					down;
		private int							len	= 6;
		private DataSeries					up;
		
		double f88, f90;
        double f0, v4, v8, vC, v10, v14, v18, v20;
        double f8, f10, f18, f20, f28, f30, f38, f48, v1C;
        double f50, f58, f60, f68, f70, f78, f80, f40;
		
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.DodgerBlue, "RSX"));
			Plots[0].Pen.Width = 2;
			
			Add(new Plot(Color.Transparent, PlotStyle.Line, ".panel range min"));            
			Add(new Plot(Color.Transparent, PlotStyle.Line, ".panel range max"));
			
			Add(new Line(Color.DarkGray, -50, "Lower Line"));
			Add(new Line(Color.DarkGray, 50, "Upper Line"));
			Add(new Line(Color.DarkGray, 0, "Mid Line"));
			
			Lines[0].Pen.DashStyle = DashStyle.Dash;
			Lines[1].Pen.DashStyle = DashStyle.Dash;
			Lines[2].Pen.DashStyle = DashStyle.Dash;
			
			

			priceSeries				= new DataSeries(this);

			CalculateOnBarClose	= false;
			Overlay				= false;
			PriceTypeSupported	= true;
		}

		/// <summary>
		/// Calculates the indicator value(s) at the current index.
		/// </summary>
		protected override void OnBarUpdate()
		{
		   
		
            if (CurrentBar == 0)
			{
                priceSeries.Set(0);
				f90=0;
				v20=0;
				v14=0;
				f88=0;
				f8=0;
				f20=0;
				f28=0;
				f18=0;
				f38=0;
				f40=0;
				f50=0;
				f60=0;
				f68=0;
				f70=0;
				f78=0;
				f80=0;
				f0=0;

				return;
			}
			
			int limit=CurrentBar-(Len)-1;


            for (int shift = limit; shift >= 0; shift--)
            {
                if (f90 == 0.0)
                {

                    f90 = 1.0;
                    f0 = 0.0;
                    if (Len - 1 >= 5) f88 = Len - 1.0; else f88 = 5.0;
                    f8 = 100.0 * (Close[shift]);
                    f18 = 3.0 / (Len + 2.0);
                    f20 = 1.0 - f18;

                }
                else
                {
                    if (f88 <= f90) f90 = f88 + 1; else f90 = f90 + 1;
                    f10 = f8;
                    f8 = 100 * Close[shift];
                    v8 = f8 - f10;
                    f28 = f20 * f28 + f18 * v8;
                    f30 = f18 * f28 + f20 * f30;
                    vC = f28 * 1.5 - f30 * 0.5;
                    f38 = f20 * f38 + f18 * vC;
                    f40 = f18 * f38 + f20 * f40;
                    v10 = f38 * 1.5 - f40 * 0.5;
                    f48 = f20 * f48 + f18 * v10;
                    f50 = f18 * f48 + f20 * f50;
                    v14 = f48 * 1.5 - f50 * 0.5;
                    f58 = f20 * f58 + f18 * Math.Abs(v8);
                    f60 = f18 * f58 + f20 * f60;
                    v18 = f58 * 1.5 - f60 * 0.5;
                    f68 = f20 * f68 + f18 * v18;

                    f70 = f18 * f68 + f20 * f70;
                    v1C = f68 * 1.5 - f70 * 0.5;
                    f78 = f20 * f78 + f18 * v1C;
                    f80 = f18 * f78 + f20 * f80;
                    v20 = f78 * 1.5 - f80 * 0.5;

                    if ((f88 >= f90) && (f8 != f10)) f0 = 1.0;
                    if ((f88 == f90) && (f0 == 0.0)) f90 = 0.0;
                }

                if ((f88 < f90) && (v20 > 0.0000000001))
                {

                    v4 = (v14 / v20 + 1.0) * 50.0;
                    if (v4 > 100.0) v4 = 100.0;
                    if (v4 < 0.0) v4 = 0.0;

                }
                else
                {
                    v4 = 50.0;
                }

				//Value.Set(v4);	
				Value.Set(-150 + (3 * v4));
            }


			#region Panel Stabilizer
				panel_range_min.Set(-1);
				panel_range_max.Set(101);
			#endregion

        }

		#region Properties

		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Default
		{
			get { return Values[0]; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Numbers of bars used for calculations")]
		[Category("Parameters")]
		public int Len
		{
			get { return len; }
			set { len = Math.Max(2, value); }
		}
		
		[Browsable(false)]	//  do not remove
        [XmlIgnore()]		//  do not remove
        public DataSeries panel_range_min
        {
            get { return Values[2]; }
        }

		[Browsable(false)]	//  do not remove
        [XmlIgnore()]		//  do not remove
        public DataSeries panel_range_max
        {
            get { return Values[1]; }
        }

		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private RSX[] cacheRSX = null;

        private static RSX checkRSX = new RSX();

        /// <summary>
        /// The RSX is a smooth, low lag price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        public RSX RSX(int len)
        {
            return RSX(Input, len);
        }

        /// <summary>
        /// The RSX is a smooth, low lag price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        public RSX RSX(Data.IDataSeries input, int len)
        {
            checkRSX.Len = len;
            len = checkRSX.Len;

            if (cacheRSX != null)
                for (int idx = 0; idx < cacheRSX.Length; idx++)
                    if (cacheRSX[idx].Len == len && cacheRSX[idx].EqualsInput(input))
                        return cacheRSX[idx];

            RSX indicator = new RSX();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Len = len;
            indicator.SetUp();

            RSX[] tmp = new RSX[cacheRSX == null ? 1 : cacheRSX.Length + 1];
            if (cacheRSX != null)
                cacheRSX.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheRSX = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The RSX is a smooth, low lag price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RSX RSX(int len)
        {
            return _indicator.RSX(Input, len);
        }

        /// <summary>
        /// The RSX is a smooth, low lag price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        public Indicator.RSX RSX(Data.IDataSeries input, int len)
        {
            return _indicator.RSX(input, len);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The RSX is a smooth, low lag price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RSX RSX(int len)
        {
            return _indicator.RSX(Input, len);
        }

        /// <summary>
        /// The RSX is a smooth, low lag price-following oscillator that ranges between 0 and 100.
        /// </summary>
        /// <returns></returns>
        public Indicator.RSX RSX(Data.IDataSeries input, int len)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.RSX(input, len);
        }

    }
}
#endregion
